/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks;

import java.util.EnumMap;
import java.util.Random;

import net.minecraft.block.Block;
import net.minecraft.block.BlockSlab;
import net.minecraft.block.SoundType;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.properties.PropertyEnum;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.IStringSerializable;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

import net.dries007.tfc.objects.Rock;
import net.dries007.tfc.objects.Wood;
import net.dries007.tfc.objects.blocks.stone.BlockRockVariant;
import net.dries007.tfc.objects.blocks.wood.BlockPlanksTFC;
import net.dries007.tfc.util.InsertOnlyEnumTable;
import net.dries007.tfc.util.OreDictionaryHelper;

public abstract class BlockSlabTFC extends BlockSlab
{
    public static final PropertyEnum<Variant> VARIANT = PropertyEnum.func_177709_a("variant", Variant.class);
    public final Block modelBlock;
    protected Half halfSlab;

    private BlockSlabTFC(Rock rock, Rock.Type type)
    {
        this(BlockRockVariant.get(rock, type));
        Block c = BlockRockVariant.get(rock, type);
        //noinspection ConstantConditions
        setHarvestLevel(c.getHarvestTool(c.func_176223_P()), c.getHarvestLevel(c.func_176223_P()));
    }

    private BlockSlabTFC(Wood wood)
    {
        this(BlockPlanksTFC.get(wood));
        Block c = BlockPlanksTFC.get(wood);
        //noinspection ConstantConditions
        setHarvestLevel(c.getHarvestTool(c.func_176223_P()), c.getHarvestLevel(c.func_176223_P()));
        Blocks.field_150480_ab.func_180686_a(this, 5, 20);
    }

    private BlockSlabTFC(Block block)
    {
        super(block.func_176223_P().func_185904_a());
        IBlockState state = field_176227_L.func_177621_b();
        if (!func_176552_j()) state = state.func_177226_a(field_176554_a, BlockSlab.EnumBlockHalf.BOTTOM);
        func_180632_j(state.func_177226_a(VARIANT, Variant.DEFAULT));
        this.modelBlock = block;
        func_149713_g(255);
    }

    @Override
    public String func_150002_b(int meta)
    {
        return func_149739_a();
    }

    @SuppressWarnings("deprecation")
    @Override
    public IBlockState func_176203_a(int meta)
    {
        IBlockState iblockstate = this.func_176223_P().func_177226_a(VARIANT, Variant.DEFAULT);

        if (!this.func_176552_j())
        {
            iblockstate = iblockstate.func_177226_a(field_176554_a, (meta & 8) == 0 ? BlockSlab.EnumBlockHalf.BOTTOM : BlockSlab.EnumBlockHalf.TOP);
        }

        return iblockstate;
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        int i = 0;

        if (!this.func_176552_j() && state.func_177229_b(field_176554_a) == BlockSlab.EnumBlockHalf.TOP)
        {
            i |= 8;
        }

        return i;
    }

    @SuppressWarnings("deprecation")
    @Override
    public float func_176195_g(IBlockState blockState, World worldIn, BlockPos pos)
    {
        return modelBlock.func_176195_g(blockState, worldIn, pos);
    }

    @Override
    public Item func_180660_a(IBlockState state, Random rand, int fortune)
    {
        return Item.func_150898_a(halfSlab);
    }

    @SuppressWarnings("deprecation")
    @Override
    public float func_149638_a(Entity exploder)
    {
        return modelBlock.func_149638_a(exploder);
    }

    @SuppressWarnings("deprecation")
    @Override
    public ItemStack func_185473_a(World worldIn, BlockPos pos, IBlockState state)
    {
        return new ItemStack(halfSlab);
    }

    @Override
    protected BlockStateContainer func_180661_e()
    {
        return this.func_176552_j() ? new BlockStateContainer(this, VARIANT) : new BlockStateContainer(this, field_176554_a, VARIANT);
    }

    @SuppressWarnings("deprecation")
    @Override
    public SoundType func_185467_w()
    {
        return modelBlock.func_185467_w();
    }

    @Override
    public IProperty<?> func_176551_l()
    {
        return VARIANT; // why is this not null-tolerable ...
    }

    @Override
    public Comparable<?> func_185674_a(ItemStack stack)
    {
        return Variant.DEFAULT;
    }

    public enum Variant implements IStringSerializable
    {
        DEFAULT;

        public String func_176610_l()
        {
            return "default";
        }
    }

    public static class Double extends BlockSlabTFC
    {
        private static final InsertOnlyEnumTable<Rock, Rock.Type, Double> ROCK_TABLE = new InsertOnlyEnumTable<>(Rock.class, Rock.Type.class);
        private static final EnumMap<Wood, Double> WOOD_MAP = new EnumMap<>(Wood.class);

        public static Double get(Rock rock, Rock.Type type)
        {
            return ROCK_TABLE.get(rock, type);
        }

        public static Double get(Wood wood)
        {
            return WOOD_MAP.get(wood);
        }

        public Double(Rock rock, Rock.Type type)
        {
            super(rock, type);
            ROCK_TABLE.put(rock, type, this);
            // No oredict, because no item.
        }

        public Double(Wood wood)
        {
            super(wood);
            if (WOOD_MAP.put(wood, this) != null) throw new IllegalStateException("There can only be one.");
            // No oredict, because no item.
        }

        public boolean func_176552_j()
        {
            return true;
        }
    }

    public static class Half extends BlockSlabTFC
    {
        private static final InsertOnlyEnumTable<Rock, Rock.Type, Half> ROCK_TABLE = new InsertOnlyEnumTable<>(Rock.class, Rock.Type.class);
        private static final EnumMap<Wood, Half> WOOD_MAP = new EnumMap<>(Wood.class);

        public static Half get(Rock rock, Rock.Type type)
        {
            return ROCK_TABLE.get(rock, type);
        }

        public static Half get(Wood wood)
        {
            return WOOD_MAP.get(wood);
        }

        public final Double doubleSlab;

        public Half(Rock rock, Rock.Type type)
        {
            super(rock, type);
            ROCK_TABLE.put(rock, type, this);
            doubleSlab = Double.get(rock, type);
            doubleSlab.halfSlab = this;
            halfSlab = this;
            OreDictionaryHelper.register(this, "slab");
            OreDictionaryHelper.registerRockType(this, type, rock, "slab");
        }

        public Half(Wood wood)
        {
            super(wood);
            if (WOOD_MAP.put(wood, this) != null) throw new IllegalStateException("There can only be one.");
            doubleSlab = Double.get(wood);
            doubleSlab.halfSlab = this;
            halfSlab = this;
            OreDictionaryHelper.register(this, "slab");
            OreDictionaryHelper.register(this, "slab", "wood");
            OreDictionaryHelper.register(this, "slab", "wood", wood);
        }

        public boolean func_176552_j()
        {
            return false;
        }
    }
}
